module.exports={A:{A:{"2":"hB","8":"I F E D A","164":"B"},B:{"1":"S UB M HB","513":"C O P H J K L"},C:{"1":"0 1 2 3 4 5 6 7 8 9 l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","8":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i gB oB","66":"j k"},D:{"1":"0 1 2 3 4 5 6 7 8 9 o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","8":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n"},E:{"1":"B C O P U x iB jB","8":"G V I F ZB SB bB cB","289":"E D A dB eB TB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","8":"D B C H J K L W X Y Z a kB lB mB nB U VB PC x"},G:{"1":"0B 1B 2B 3B 4B 5B 6B 7B 8B","8":"SB qB XB sB tB uB","289":"E vB wB xB yB zB"},H:{"2":"9B"},I:{"1":"M","8":"OB G AC BC CC DC XB EC FC"},J:{"8":"F A"},K:{"1":"Q","8":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"8":"A","164":"B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:2,C:"Web Cryptography"};
