module.exports={A:{A:{"1":"A B","2":"I F hB","132":"E D"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"G V I F E D A B C O P ZB SB bB cB dB eB TB U x iB jB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N U VB PC x","2":"D kB lB mB nB"},G:{"1":"SB qB XB sB","513":"E tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"4097":"9B"},I:{"1025":"OB G M AC BC CC DC XB EC FC"},J:{"258":"F A"},K:{"2":"A","258":"B C Q U VB x"},L:{"1025":"M"},M:{"2049":"N"},N:{"258":"A B"},O:{"258":"GC"},P:{"1025":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1025":"OC"},S:{"1":"fB"}},B:1,C:"Basic console logging functions"};
