module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"P H J K L","2":"C O","257":"S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h gB oB","194":"i j k l m n o p q r s t u v w Q y z"},D:{"1":"0 1 2 3 4 5 k l m n o p q r s t u v w Q y z","2":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j","257":"6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"F E D A B C O P dB eB TB U x iB jB","2":"G V I ZB SB bB cB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 e f g h i j k l m n o p q r s t u v w Q y z BB DB EB","2":"D B C H J K L W X Y Z a b c d kB lB mB nB U VB PC x","257":"CB AB T R N"},G:{"1":"E uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB sB tB"},H:{"2":"9B"},I:{"2":"OB G M AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"2":"A B C Q U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"1":"HC IC JC KC LC TB MC","2":"G"},Q:{"1":"NC"},R:{"2":"OC"},S:{"1":"fB"}},B:7,C:"Speech Synthesis API"};
