module.exports={A:{A:{"2":"hB","8":"I F E D A","129":"B"},B:{"1":"S UB M HB","129":"C O P H J K L"},C:{"1":"0 1 2 3 4 5 6 7 8 9 b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB oB","129":"G V I F E D A B C O P H J K L W X Y Z a"},D:{"1":"0 1 2 3 4 5 6 7 8 9 k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F","129":"E D A B C O P H J K L W X Y Z a b c d e f g h i j"},E:{"1":"E D A B C O P eB TB U x iB jB","2":"G V ZB SB","129":"I F bB cB dB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B kB lB mB nB U VB PC","129":"C H J K L x"},G:{"1":"E vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB sB tB uB"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"1":"A","2":"F"},K:{"1":"C Q x","2":"A B U VB"},L:{"1":"M"},M:{"1":"N"},N:{"8":"A","129":"B"},O:{"129":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"129":"fB"}},B:6,C:"WebGL - 3D Canvas graphics"};
