module.exports={A:{A:{"2":"I F hB","4":"E D A B"},B:{"4":"C O P H J K L S UB M HB"},C:{"4":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB"},D:{"4":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"ZB SB","4":"G V I F E D A B C O P bB cB dB eB TB U x iB jB"},F:{"2":"D","4":"0 1 2 3 4 5 6 7 8 9 B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N kB lB mB nB U VB PC x"},G:{"4":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"4":"9B"},I:{"2":"OB G AC BC CC DC XB","4":"M EC FC"},J:{"2":"F A"},K:{"4":"A B C Q U VB x"},L:{"4":"M"},M:{"4":"N"},N:{"4":"A B"},O:{"2":"GC"},P:{"4":"G HC IC JC KC LC TB MC"},Q:{"4":"NC"},R:{"4":"OC"},S:{"4":"fB"}},B:2,C:"WAI-ARIA Accessibility features"};
