module.exports={A:{A:{"1":"A B","2":"I F E D hB"},B:{"1":"S UB M HB","2":"C O P H J K L"},C:{"1":"0 1 2 3 4 5 6 7 8 9 o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB oB","132":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n"},D:{"1":"0 1 2 3 4 5 6 7 8 9 d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G","16":"V I F E Y Z a b c","132":"D A B C O P H J K L W X"},E:{"1":"C O P U x iB jB","2":"G V ZB SB bB","132":"I F E D A B cB dB eB TB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x"},G:{"2":"tB uB","132":"E vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","514":"SB qB XB sB"},H:{"2":"9B"},I:{"2":"AC BC CC","260":"OB G DC XB","514":"M EC FC"},J:{"132":"A","260":"F"},K:{"2":"A B C U VB x","260":"Q"},L:{"260":"M"},M:{"2":"N"},N:{"514":"A","1028":"B"},O:{"2":"GC"},P:{"260":"G HC IC JC KC LC TB MC"},Q:{"260":"NC"},R:{"260":"OC"},S:{"1":"fB"}},B:1,C:"accept attribute for file input"};
