module.exports={A:{A:{"2":"D A B hB","8":"I F E"},B:{"2":"C O P H J K L S UB M HB"},C:{"2":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB"},D:{"1":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o","2":"2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","130":"0 1 p q r s t u v w Q y z"},E:{"1":"G V I F E D A B C O P SB bB cB dB eB TB U x iB jB","2":"ZB"},F:{"1":"D B C H J K L W X Y Z a b kB lB mB nB U VB PC x","2":"0 1 2 3 4 5 6 7 8 9 o p q r s t u v w Q y z BB DB EB CB AB T R N","130":"c d e f g h i j k l m n"},G:{"1":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"258":"9B"},I:{"1":"OB G DC XB EC FC","2":"M AC BC CC"},J:{"1":"F A"},K:{"1":"A B C Q U VB x"},L:{"130":"M"},M:{"2":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"1":"G","130":"HC IC JC KC LC TB MC"},Q:{"2":"NC"},R:{"130":"OC"},S:{"2":"fB"}},B:2,C:"SVG fonts"};
