module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"S","2":"C O P H J K L UB M HB"},C:{"2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB","66":"0 1 2 3 4 5 6 7 8 9 g h i j k l m n o p q r s t u v w Q y z WB BB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"G V I F E D A B C O P H J K L W X Y Z a b UB M HB pB rB YB","33":"c d e f g h i j k l"},E:{"2":"G V I F E D A B C O P ZB SB bB cB dB eB TB U x iB jB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T","2":"D B C R N kB lB mB nB U VB PC x","33":"H J K L W X Y"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB G M AC BC CC DC XB","33":"EC FC"},J:{"2":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"2":"M"},M:{"2":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","33":"G"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:7,C:"Shadow DOM (deprecated V0 spec)"};
