module.exports={A:{A:{"2":"I F E D hB","6308":"A","6436":"B"},B:{"1":"S UB M HB","6436":"C O P H J K L"},C:{"1":"N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p gB oB","2052":"0 1 2 3 4 5 6 7 8 9 q r s t u v w Q y z WB BB GB DB EB CB AB T R"},D:{"1":"IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB","8258":"T R N"},E:{"1":"B C O P U x iB jB","2":"G V I F E ZB SB bB cB dB","3108":"D A eB TB"},F:{"1":"CB AB T R N","2":"0 1 2 3 4 D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z kB lB mB nB U VB PC x","8258":"5 6 7 8 9 BB DB EB"},G:{"1":"0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB","3108":"wB xB yB zB"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"2":"A B C Q U VB x"},L:{"1":"M"},M:{"2052":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"1":"TB MC","2":"G HC IC JC KC LC"},Q:{"2":"NC"},R:{"2":"OC"},S:{"2052":"fB"}},B:4,C:"CSS Scroll Snap"};
