module.exports={A:{A:{"2":"I F E D A hB","130":"B"},B:{"16":"C O","260":"P H","1028":"S UB M HB","5124":"J K L"},C:{"2":"0 1 2 3 4 5 6 7 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z gB oB","6148":"AB T R N IB JB KB LB MB NB FB PB QB RB S","6722":"8 9 WB BB GB DB EB CB"},D:{"2":"0 1 2 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","260":"3 4 5 6 7 8 9","1028":"WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G V I F E D ZB SB bB cB dB eB","3076":"A B C O P TB U x iB jB"},F:{"2":"D B C H J K L W X Y Z a b c d e f g h i j k l m n o p kB lB mB nB U VB PC x","260":"q r s t u v w","1028":"0 1 2 3 4 5 6 7 8 9 Q y z BB DB EB CB AB T R N"},G:{"2":"E SB qB XB sB tB uB vB wB xB","16":"yB","1028":"zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB EC FC","260":"M"},J:{"2":"F A"},K:{"2":"A B C U VB x","1028":"Q"},L:{"1028":"M"},M:{"2626":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"2":"G HC IC","1028":"JC KC LC TB MC"},Q:{"1028":"NC"},R:{"2":"OC"},S:{"2":"fB"}},B:1,C:"Streams"};
