module.exports={A:{A:{"1":"I F E D A B hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","4":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u"},E:{"1":"D A B C O P eB TB U x iB jB","4":"G V I F E ZB SB bB cB dB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x","4":"H J K L W X Y Z a b c d e f g h"},G:{"1":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","4":"E SB qB XB sB tB uB vB"},H:{"2":"9B"},I:{"1":"M","4":"OB G AC BC CC DC XB EC FC"},J:{"4":"F A"},K:{"2":"A B C U VB x","4":"Q"},L:{"1":"M"},M:{"1":"N"},N:{"1":"A B"},O:{"4":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"4":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:5,C:"CSS3 word-break"};
