module.exports={A:{A:{"2":"I F E hB","260":"D A B"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P gB oB"},D:{"1":"1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m","132":"0 n o p q r s t u v w Q y z"},E:{"1":"C O P U x iB jB","2":"G V I F D A B ZB SB bB cB eB TB","132":"E dB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 o p q r s t u v w Q y z BB DB EB CB AB T R N x","2":"H J K L W X Y Z","4":"B C lB mB nB U VB PC","16":"D kB","132":"a b c d e f g h i j k l m n"},G:{"1":"1B 2B 3B 4B 5B 6B 7B 8B","2":"SB qB XB sB tB uB wB xB yB zB 0B","132":"E vB"},H:{"1":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F","132":"A"},K:{"1":"Q x","4":"A B C U VB"},L:{"1":"M"},M:{"1":"N"},N:{"1":"A B"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","132":"G"},Q:{"1":"NC"},R:{"132":"OC"},S:{"1":"fB"}},B:4,C:"SVG fragment identifiers"};
