module.exports={A:{A:{"1":"D A B","2":"hB","8":"I F E"},B:{"1":"C O P H J K L","129":"S UB M HB"},C:{"1":"0 1 2 3 4 5 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z gB oB","8":"aB OB","129":"6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"1":"0 V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","4":"G","129":"1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"V I F E D B C O P bB cB dB eB TB U x iB jB","8":"G ZB SB","129":"A"},F:{"1":"B C J K L W X Y Z a b c d e f g h i j k l m n o p nB U VB PC x","2":"D H kB","8":"lB mB","129":"0 1 2 3 4 5 6 7 8 9 q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"1":"E SB qB XB sB tB uB vB wB xB","129":"yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"OB G AC BC CC DC XB EC FC","129":"M"},J:{"1":"F A"},K:{"1":"B C Q U VB x","8":"A"},L:{"129":"M"},M:{"129":"N"},N:{"1":"A B"},O:{"1":"GC"},P:{"1":"G","129":"HC IC JC KC LC TB MC"},Q:{"129":"NC"},R:{"129":"OC"},S:{"1":"fB"}},B:2,C:"Geolocation"};
