module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H gB oB","132":"J K L W X Y Z a b","260":"c d e f g h","516":"i"},D:{"1":"0 1 2 3 4 5 6 7 8 9 l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L","1028":"W X Y Z a b c d e f g h i j k"},E:{"1":"D A B C O P eB TB U x iB jB","2":"G V I F E ZB SB bB cB dB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x","1028":"H J K L W X"},G:{"1":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC","1028":"DC XB EC FC"},J:{"2":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:6,C:"ES6 Number"};
