module.exports={A:{A:{"1":"B","2":"I F hB","132":"A","260":"E D"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB KB LB MB NB FB PB QB RB S gB oB","2":"aB OB","1025":"GB DB EB CB AB T R N IB JB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","132":"G V I F E D A B C"},E:{"2":"ZB SB","513":"I F E D A B C O P cB dB eB TB U x iB jB","644":"G V bB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N x","2":"D B kB lB mB nB U VB PC"},G:{"513":"E tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","644":"SB qB XB sB"},H:{"2":"9B"},I:{"1":"M EC FC","132":"OB G AC BC CC DC XB"},J:{"1":"A","132":"F"},K:{"1":"C Q x","2":"A B U VB"},L:{"1":"M"},M:{"1":"N"},N:{"1":"B","132":"A"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:1,C:"Cross-Origin Resource Sharing"};
