module.exports={A:{A:{"132":"I F E D A B hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m gB oB","322":"n o p q r"},D:{"1":"0 1 2 3 4 5 6 7 8 9 z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I","16":"F","33":"E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y"},E:{"1":"B C O P U x iB jB","2":"G ZB SB","16":"V","33":"I F E D A bB cB dB eB TB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x","33":"H J K L W X Y Z a b c d e f g h i j k l"},G:{"1":"0B 1B 2B 3B 4B 5B 6B 7B 8B","16":"SB qB XB","33":"E sB tB uB vB wB xB yB zB"},H:{"2":"9B"},I:{"1":"M","2":"AC BC CC","33":"OB G DC XB EC FC"},J:{"33":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"36":"A B"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","33":"G"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:4,C:"CSS writing-mode property"};
