module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"S UB M HB","2":"C O P H J K L"},C:{"1":"1 2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","16":"aB","33":"0 OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z gB oB"},D:{"1":"AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","16":"G V I F E D A B C O P","33":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB"},E:{"1":"D A B C O P eB TB U x iB jB","16":"G V I ZB SB bB","33":"F E cB dB"},F:{"1":"3 4 5 6 7 8 9 BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x","33":"0 1 2 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},G:{"1":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","16":"SB qB XB sB","33":"E tB uB vB"},H:{"2":"9B"},I:{"1":"M","16":"OB G AC BC CC DC XB","33":"EC FC"},J:{"16":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"33":"GC"},P:{"1":"LC TB MC","16":"G","33":"HC IC JC KC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"33":"fB"}},B:5,C:"CSS :any-link selector"};
