module.exports={A:{A:{"1":"B","16":"hB","132":"I F E D A"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","132":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f gB oB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","132":"G V I F E D A B C O P H J K L W X Y Z a"},E:{"1":"A B C O P TB U x iB jB","132":"G V I F E D ZB SB bB cB dB eB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","16":"D B C kB lB mB nB U VB PC","132":"x"},G:{"1":"yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","132":"E SB qB XB sB tB uB vB wB xB"},H:{"132":"9B"},I:{"1":"M EC FC","132":"OB G AC BC CC DC XB"},J:{"132":"F A"},K:{"1":"Q","16":"A B C U VB","132":"x"},L:{"1":"M"},M:{"1":"N"},N:{"1":"B","132":"A"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","132":"G"},Q:{"1":"NC"},R:{"1":"OC"},S:{"4":"fB"}},B:6,C:"localeCompare()"};
