module.exports={A:{A:{"1":"B","4":"I F E D A hB"},B:{"1":"C O P H J","129":"K L S UB M HB"},C:{"1":"0 1 I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","4":"aB OB G V gB oB","129":"2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"1":"0 1 2 3 4 5 w Q y z","4":"G V I","129":"6 7 8 9 F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"4":"G V ZB SB","129":"I F E D A B C O P bB cB dB eB TB U x iB jB"},F:{"1":"C j k l m n o p q r s U VB PC x","4":"D B kB lB mB nB","129":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i t u v w Q y z BB DB EB CB AB T R N"},G:{"4":"SB qB XB","129":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"4":"9B"},I:{"4":"AC BC CC","129":"OB G M DC XB EC FC"},J:{"129":"F A"},K:{"1":"C U VB x","4":"A B","129":"Q"},L:{"129":"M"},M:{"129":"N"},N:{"1":"B","4":"A"},O:{"129":"GC"},P:{"129":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"129":"OC"},S:{"1":"fB"}},B:1,C:"dataset & data-* attributes"};
