module.exports={A:{A:{"2":"I F E D A hB","388":"B"},B:{"1":"L S UB M HB","2":"C O P H","129":"J K"},C:{"1":"BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB gB oB"},D:{"1":"2 3 4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"0 1 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","513":"UB M HB pB rB YB"},E:{"2":"G V I F E D A B ZB SB bB cB dB eB TB U","2052":"P iB jB","3076":"C O x"},F:{"1":"0 1 2 3 4 5 6 7 8 9 q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C H J K L W X Y Z a b c d e f g h i j k l m n o p kB lB mB nB U VB PC x"},G:{"1":"4B 5B 6B 7B 8B","2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B","2052":"2B 3B"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"1":"Q","2":"A B C U VB x"},L:{"513":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"2":"GC"},P:{"1":"HC IC JC KC LC TB MC","2":"G"},Q:{"16":"NC"},R:{"1":"OC"},S:{"2":"fB"}},B:6,C:"'SameSite' cookie attribute"};
