module.exports={A:{A:{"4":"I F E D A B hB"},B:{"4":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","8":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o gB oB"},D:{"4":"0 1 2 3 4 5 6 7 8 9 V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","8":"G"},E:{"4":"V I F E D A B C O P bB cB dB eB TB U x iB jB","8":"G ZB SB"},F:{"4":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","8":"D B C kB lB mB nB U VB PC x"},G:{"4":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","8":"SB qB XB"},H:{"8":"9B"},I:{"4":"OB G M DC XB EC FC","8":"AC BC CC"},J:{"4":"A","8":"F"},K:{"4":"Q","8":"A B C U VB x"},L:{"4":"M"},M:{"1":"N"},N:{"4":"A B"},O:{"4":"GC"},P:{"4":"G HC IC JC KC LC TB MC"},Q:{"4":"NC"},R:{"4":"OC"},S:{"1":"fB"}},B:1,C:"Ruby annotation"};
