module.exports={A:{A:{"2":"I F E D A B hB"},B:{"1":"S UB M HB","2":"C O P H J K L"},C:{"2":"aB OB gB oB","33":"4 5 6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","164":"0 1 2 3 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z"},D:{"1":"0 1 2 3 4 5 6 7 8 9 t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L W X","132":"Y Z a b c d e f g h i j k l m n o p q r s"},E:{"1":"P iB jB","2":"G V I ZB SB bB","132":"F E D A B C O cB dB eB TB U x"},F:{"1":"0 1 2 3 4 5 6 7 8 9 g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D kB lB mB","132":"H J K L W X Y Z a b c d e f","164":"B C nB U VB PC x"},G:{"1":"7B 8B","2":"SB qB XB sB tB","132":"E uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B"},H:{"164":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB","132":"EC FC"},J:{"132":"F A"},K:{"1":"Q","2":"A","164":"B C U VB x"},L:{"1":"M"},M:{"33":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"164":"fB"}},B:5,C:"CSS3 tab-size"};
