module.exports={A:{A:{"2":"I F E D A B hB"},B:{"2":"C O P H J K L","2052":"S UB M HB"},C:{"2":"aB OB G V gB oB","1028":"0 1 2 3 4 5 6 7 8 9 n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","1060":"I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m"},D:{"2":"G V I F E D A B C O P H J K L W X Y Z a b c","226":"0 1 2 3 4 5 6 7 d e f g h i j k l m n o p q r s t u v w Q y z","2052":"8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G V I F ZB SB bB cB","772":"O P x iB jB","804":"E D A B C eB TB U","1316":"dB"},F:{"2":"D B C H J K L W X Y Z a b c d e f g h i j k l kB lB mB nB U VB PC x","226":"m n o p q r s t u","2052":"0 1 2 3 4 5 6 7 8 9 v w Q y z BB DB EB CB AB T R N"},G:{"2":"SB qB XB sB tB uB","292":"E vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"2":"A B C U VB x","2052":"Q"},L:{"2052":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"2052":"GC"},P:{"2":"G HC IC","2052":"JC KC LC TB MC"},Q:{"2":"NC"},R:{"1":"OC"},S:{"1028":"fB"}},B:4,C:"text-decoration styling"};
