module.exports={A:{A:{"2":"hB","8":"I F E","260":"D A B"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB","132":"OB gB oB","260":"G V I F E D A B C O P H J K L W X"},D:{"1":"0 1 2 3 4 5 6 7 8 9 d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","132":"G V","260":"I F E D A B C O P H J K L W X Y Z a b c"},E:{"1":"F E D A B C O P cB dB eB TB U x iB jB","132":"G ZB SB","260":"V I bB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","132":"D B kB lB mB nB","260":"C U VB PC x"},G:{"1":"E uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","132":"SB","260":"qB XB sB tB"},H:{"132":"9B"},I:{"1":"M EC FC","132":"AC","260":"OB G BC CC DC XB"},J:{"260":"F A"},K:{"1":"Q","132":"A","260":"B C U VB x"},L:{"1":"M"},M:{"1":"N"},N:{"260":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:1,C:"HTML5 semantic elements"};
