module.exports={A:{A:{"1":"D A B","2":"I F E hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S gB oB","16":"aB"},D:{"1":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"G V I F E D A B C O P bB cB dB eB TB U x iB jB","16":"ZB SB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N x","2":"D B kB lB mB nB U VB PC","16":"C"},G:{"1":"E sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","16":"SB qB XB"},H:{"2":"9B"},I:{"1":"OB G M CC DC XB EC FC","16":"AC BC"},J:{"1":"F A"},K:{"1":"x","2":"A B U VB","16":"C","130":"Q"},L:{"1":"M"},M:{"130":"N"},N:{"130":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:7,C:"KeyboardEvent.charCode"};
