module.exports={A:{A:{"2":"I F E D A B hB"},B:{"2":"C O P H J K L","260":"S UB M HB"},C:{"2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j gB oB","260":"WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","516":"0 1 2 3 4 5 6 7 8 9 y z","580":"k l m n o p q r s t u v w Q"},D:{"2":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m","132":"n o p","260":"0 1 2 3 4 5 6 7 8 9 q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G V I F E D A ZB SB bB cB dB eB TB","1090":"B C O U x","2049":"P iB jB"},F:{"2":"D B C H J K L W X Y Z kB lB mB nB U VB PC x","132":"a b c","260":"0 1 2 3 4 5 6 7 8 9 d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB zB","1090":"0B 1B 2B 3B 4B 5B 6B","2049":"7B 8B"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB EC FC","260":"M"},J:{"2":"F A"},K:{"2":"A B C U VB x","260":"Q"},L:{"260":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"260":"GC"},P:{"260":"G HC IC JC KC LC TB MC"},Q:{"260":"NC"},R:{"260":"OC"},S:{"516":"fB"}},B:5,C:"Web Animations API"};
