module.exports={A:{A:{"2":"I F E hB","8":"D A B"},B:{"1":"S UB M HB","4":"P H J K L","8":"C O"},C:{"1":"0 1 2 3 4 5 6 7 8 9 f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB oB","4":"G V I F E D A B C O P H J K L W X Y Z a b c d e"},D:{"1":"0 1 2 3 4 5 6 7 8 9 c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V","4":"I F E D A B C O P H J K L W X Y Z a b"},E:{"2":"ZB","8":"G V I F E D A B C SB bB cB dB eB TB U","132":"O P x iB jB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D kB lB mB","4":"B C H nB U VB PC x"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B","132":"3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"M","2":"AC BC","4":"OB G CC DC XB EC FC"},J:{"2":"F A"},K:{"2":"A B C U VB x","4":"Q"},L:{"1":"M"},M:{"1":"N"},N:{"8":"A B"},O:{"1":"GC"},P:{"1":"HC IC JC KC LC TB MC","4":"G"},Q:{"1":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:6,C:"WebM video format"};
