module.exports={A:{A:{"2":"I F E D hB","8":"A B"},B:{"1":"S UB M HB","578":"C O P H J K L"},C:{"1":"3 4 5 6 7 8 9 L W X Y Z a b WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB oB","4":"G V I F E D A B C O P H J K","194":"0 1 2 c d e f g h i j k l m n o p q r s t u v w Q y z"},D:{"1":"0 1 2 3 4 5 6 7 8 9 Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P H J K L W X Y"},E:{"2":"G V I F E D A B C O P ZB SB bB cB dB eB TB U x iB jB"},F:{"1":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N","2":"D B C kB lB mB nB U VB PC x"},G:{"1":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"OB G M AC BC CC DC XB EC FC"},J:{"1":"F A"},K:{"1":"B C Q U VB x","2":"A"},L:{"1":"M"},M:{"1":"N"},N:{"8":"A","260":"B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"1":"NC"},R:{"1":"OC"},S:{"2":"fB"}},B:2,C:"Touch events"};
