module.exports={A:{A:{"2":"hB","8":"I F E D","260":"A B"},B:{"1":"S UB M HB","260":"C O P H","1284":"J K L"},C:{"8":"aB OB gB oB","4612":"0 1 2 3 4 5 6 7 8 9 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"1":"IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","8":"G V I F E D A B C O P H J K L W","132":"0 1 2 3 4 5 6 7 8 9 X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N"},E:{"1":"O P x iB jB","8":"G V I F E D A B C ZB SB bB cB dB eB TB U"},F:{"1":"D B C CB AB T R N kB lB mB nB U VB PC x","132":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB"},G:{"8":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B 2B","2049":"3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"FC","8":"OB G AC BC CC DC XB EC","132":"M"},J:{"1":"A","8":"F"},K:{"1":"A B C U VB x","8":"Q"},L:{"1":"M"},M:{"516":"N"},N:{"8":"A B"},O:{"8":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"132":"NC"},R:{"1":"OC"},S:{"2":"fB"}},B:1,C:"Datalist element"};
