module.exports={A:{A:{"2":"I F E D A hB","548":"B"},B:{"1":"S UB M HB","516":"C O P H J K L"},C:{"1":"CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D gB oB","676":"A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q","1700":"0 1 2 3 4 5 6 7 8 9 y z WB BB GB DB EB"},D:{"1":"KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"G V I F E D A B C O P","676":"H J K L W","804":"0 1 2 3 4 5 6 7 8 9 X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB"},E:{"2":"G V ZB SB","676":"bB","804":"I F E D A B C O P cB dB eB TB U x iB jB"},F:{"1":"CB AB T R N x","2":"D B C kB lB mB nB U VB PC","804":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB zB 0B 1B","2052":"2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB G M AC BC CC DC XB EC FC"},J:{"2":"F","292":"A"},K:{"2":"A B C U VB x","804":"Q"},L:{"804":"M"},M:{"1":"N"},N:{"2":"A","548":"B"},O:{"804":"GC"},P:{"1":"TB MC","804":"G HC IC JC KC LC"},Q:{"804":"NC"},R:{"804":"OC"},S:{"1":"fB"}},B:1,C:"Full Screen API"};
