module.exports={A:{A:{"1":"A B","2":"I F E D hB"},B:{"1":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G gB oB","33":"V I F E D A B C O P H"},D:{"1":"0 1 2 3 4 5 6 7 8 9 u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","33":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t"},E:{"1":"D A B C O P eB TB U x iB jB","2":"ZB SB","33":"I F E bB cB dB","292":"G V"},F:{"1":"0 1 2 3 4 5 6 7 8 9 h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N x","2":"D B kB lB mB nB U VB PC","33":"C H J K L W X Y Z a b c d e f g"},G:{"1":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","33":"E tB uB vB","164":"SB qB XB sB"},H:{"2":"9B"},I:{"1":"M","33":"G DC XB EC FC","164":"OB AC BC CC"},J:{"33":"F A"},K:{"1":"Q x","2":"A B C U VB"},L:{"1":"M"},M:{"1":"N"},N:{"1":"A B"},O:{"1":"GC"},P:{"1":"G HC IC JC KC LC TB MC"},Q:{"33":"NC"},R:{"1":"OC"},S:{"1":"fB"}},B:5,C:"CSS Animation"};
