module.exports={A:{A:{"2":"I F E D hB","8":"A B"},B:{"1":"S UB M HB","8":"C O P H J K L"},C:{"1":"EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g gB oB","8":"0 h i j k l m n o p q r s t u v w Q y z","456":"1 2 3 4 5 6 7 8 9","712":"WB BB GB DB"},D:{"1":"R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","2":"0 1 2 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","8":"3 4","132":"5 6 7 8 9 WB BB GB DB EB CB AB T"},E:{"2":"G V I F ZB SB bB cB dB","8":"E D A eB","132":"B C O P TB U x iB jB"},F:{"1":"CB AB T R N","2":"D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r kB lB mB nB U VB PC x","132":"0 1 2 3 4 5 6 7 8 9 s t u v w Q y z BB DB EB"},G:{"2":"E SB qB XB sB tB uB vB wB xB yB","132":"zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"1":"M","2":"OB G AC BC CC DC XB EC FC"},J:{"2":"F A"},K:{"2":"A B C U VB x","132":"Q"},L:{"1":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"1":"GC"},P:{"1":"IC JC KC LC TB MC","2":"G","132":"HC"},Q:{"132":"NC"},R:{"132":"OC"},S:{"8":"fB"}},B:1,C:"Custom Elements (V1)"};
