module.exports={A:{A:{"2":"I F E D hB","130":"A B"},B:{"130":"C O P H J K L S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB gB oB","130":"G V I F E D A B C O P H J K L W X Y Z a","322":"b c d e f g h i j k"},D:{"2":"G V I F E D A B C O P H","130":"0 1 2 3 4 5 6 7 8 9 J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"1":"A B C O P eB TB U x iB jB","2":"F E D ZB SB cB dB","130":"G V I bB"},F:{"2":"D B C kB lB mB nB U VB PC x","130":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"1":"xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","2":"E SB uB vB wB","130":"qB XB sB tB"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB","130":"M EC FC"},J:{"2":"F","130":"A"},K:{"2":"A B C U VB x","130":"Q"},L:{"130":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"130":"GC"},P:{"130":"G HC IC JC KC LC TB MC"},Q:{"130":"NC"},R:{"130":"OC"},S:{"1":"fB"}},B:5,C:"CSS font-variant-alternates"};
