module.exports={A:{A:{"2":"I F E D A B hB"},B:{"2":"C O P","194":"S UB M HB","257":"H J K L"},C:{"2":"0 1 2 3 4 5 6 7 8 9 aB OB G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB gB oB","16":"RB S"},D:{"2":"G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p","16":"0 1 2 3 4 5 6 7 8 q r s t u v w Q y z","194":"9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G V I F E ZB SB bB cB dB","16":"D A B C O P eB TB U x iB jB"},F:{"2":"D B C H J K L W X Y Z a b c d e f g kB lB mB nB U VB PC x","16":"0 1 2 3 4 5 6 7 8 9 h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"2":"E SB qB XB sB tB uB vB","16":"wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"16":"9B"},I:{"2":"OB G AC BC CC DC XB EC FC","16":"M"},J:{"2":"F A"},K:{"2":"A B C U VB x","16":"Q"},L:{"16":"M"},M:{"16":"N"},N:{"2":"A","16":"B"},O:{"16":"GC"},P:{"16":"G HC IC JC KC LC TB MC"},Q:{"16":"NC"},R:{"16":"OC"},S:{"2":"fB"}},B:6,C:"Token Binding"};
