module.exports={A:{A:{"1":"E D A B","2":"I F hB"},B:{"1":"C O P H J K L","4":"S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB","4":"G V I F E D A B C O P H J K JB KB LB MB NB FB PB QB RB S","16":"aB OB gB oB"},D:{"4":"0 1 2 3 4 5 6 7 8 9 d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB","16":"G V I F E D A B C O P H J K L W X Y Z a b c"},E:{"4":"I F E D A B C O P bB cB dB eB TB U x iB jB","16":"G V ZB SB"},F:{"4":"0 1 2 3 4 5 6 7 8 9 C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N PC x","16":"D B kB lB mB nB U VB"},G:{"4":"E uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B","16":"SB qB XB sB tB"},H:{"2":"9B"},I:{"4":"G M DC XB EC FC","16":"OB AC BC CC"},J:{"4":"F A"},K:{"4":"Q x","16":"A B C U VB"},L:{"4":"M"},M:{"1":"N"},N:{"1":"A B"},O:{"4":"GC"},P:{"4":"G HC IC JC KC LC TB MC"},Q:{"4":"NC"},R:{"4":"OC"},S:{"1":"fB"}},B:6,C:"X-Frame-Options HTTP header"};
