module.exports={A:{A:{"2":"I F E D A hB","132":"B"},B:{"1":"C O P H J K L","4":"S UB M HB"},C:{"2":"aB OB gB","4":"0 1 2 3 4 5 6 7 8 9 I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","8":"G V oB"},D:{"2":"G V I","4":"0 1 2 3 4 5 6 7 8 9 F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G V I F E D A B C O P ZB SB bB cB dB eB TB U x iB jB"},F:{"2":"D B C kB lB mB nB U VB PC x","4":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"2":"SB qB","4":"E XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"AC BC CC","4":"OB G M DC XB EC FC"},J:{"2":"F","4":"A"},K:{"1":"C x","2":"A B U VB","4":"Q"},L:{"4":"M"},M:{"4":"N"},N:{"1":"B","2":"A"},O:{"4":"GC"},P:{"4":"G HC IC JC KC LC TB MC"},Q:{"4":"NC"},R:{"4":"OC"},S:{"4":"fB"}},B:4,C:"DeviceOrientation & DeviceMotion events"};
