module.exports={A:{A:{"2":"I F E hB","260":"D A B"},B:{"132":"S UB M HB","260":"C O P H J K L"},C:{"2":"aB OB G V gB oB","260":"0 1 2 3 4 5 6 7 8 9 I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S"},D:{"16":"G V I F E D A B C O P","132":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"16":"ZB SB","132":"G V I F E D A B C O P bB cB dB eB TB U x iB jB"},F:{"1":"C PC x","2":"D kB lB mB nB","16":"B U VB","132":"0 1 2 3 4 5 6 7 8 9 H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z BB DB EB CB AB T R N"},G:{"16":"SB qB","132":"E XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"16":"AC BC","132":"OB G M CC DC XB EC FC"},J:{"132":"F A"},K:{"1":"C x","2":"A","16":"B U VB","132":"Q"},L:{"132":"M"},M:{"260":"N"},N:{"260":"A B"},O:{"132":"GC"},P:{"132":"G HC IC JC KC LC TB MC"},Q:{"132":"NC"},R:{"132":"OC"},S:{"260":"fB"}},B:5,C:"Mutation events"};
