
/* BEGIN Ubuntu Mono Regular */
@font-face {
    font-family: 'Ubuntu Mono';
    src: url('fonts/ubuntumono-regular-webfont.eot');
    src: url('fonts/ubuntumono-regular-webfont.eot?#iefix') format('embedded-opentype'),
         url('fonts/ubuntumono-regular-webfont.woff2') format('woff2'),
         url('fonts/ubuntumono-regular-webfont.woff') format('woff'),
         url('fonts/ubuntumono-regular-webfont.ttf') format('truetype'),
         url('fonts/ubuntumono-regular-webfont.svg#ubuntu_monoregular') format('svg');
    font-weight: normal;
    font-style: normal;

}
/* END Ubuntu Mono Regular */

/* BEGIN Ubuntu Mono Italic */
@font-face {
    font-family: 'Ubuntu Mono';
    src: url('fonts/ubuntumono-regular-italic-webfont.eot');
    src: url('fonts/ubuntumono-regular-italic-webfont.eot?#iefix') format('embedded-opentype'),
         url('fonts/ubuntumono-regular-italic-webfont.woff2') format('woff2'),
         url('fonts/ubuntumono-regular-italic-webfont.woff') format('woff'),
         url('fonts/ubuntumono-regular-italic-webfont.ttf') format('truetype'),
         url('fonts/ubuntumono-regular-italic-webfont.svg#ubuntu_monoitalic') format('svg');
    font-weight: normal;
    font-style: italic;
}
/* END Ubuntu Mono Italic */

/* BEGIN Ubuntu Mono Bold */
@font-face {
    font-family: 'Ubuntu Mono';
    src: url('fonts/ubuntumono-bold-webfont.eot');
    src: url('fonts/ubuntumono-bold-webfont.eot?#iefix') format('embedded-opentype'),
         url('fonts/ubuntumono-bold-webfont.woff2') format('woff2'),
         url('fonts/ubuntumono-bold-webfont.woff') format('woff'),
         url('fonts/ubuntumono-bold-webfont.ttf') format('truetype'),
         url('fonts/ubuntumono-bold-webfont.svg#ubuntu_monobold') format('svg');
    font-weight: bold;
    font-style: normal;
}
/* END Ubuntu Mono Bold */

/* BEGIN Ubuntu Mono Bold Italic */
@font-face {
    font-family: 'Ubuntu Mono';
    src: url('fonts/ubuntumono-bold-italic-webfont.eot');
    src: url('fonts/ubuntumono-bold-italic-webfont.eot?#iefix') format('embedded-opentype'),
         url('fonts/ubuntumono-bold-italic-webfont.woff2') format('woff2'),
         url('fonts/ubuntumono-bold-italic-webfont.woff') format('woff'),
         url('fonts/ubuntumono-bold-italic-webfont.ttf') format('truetype'),
         url('fonts/ubuntumono-bold-italic-webfont.svg#ubuntu_monobold_italic') format('svg');
    font-weight: bold;
    font-style: italic;

}
/* END Ubuntu Mono Bold Italic */
