module.exports={A:{A:{"2":"I F E D hB","33":"A B"},B:{"33":"C O P H J K L","132":"S UB M HB"},C:{"1":"0 1 2 3 4 5 6 7 8 9 u v w Q y z WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S","2":"aB OB G V gB oB","33":"I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t"},D:{"2":"0 1 2 3 4 5 G V I F E D A B C O P H J K L W X Y Z a b c d e f g h i j k l m n o p q r s t u v w Q y z","132":"6 7 8 9 WB BB GB DB EB CB AB T R N IB JB KB LB MB NB FB PB QB RB S UB M HB pB rB YB"},E:{"2":"G V ZB SB","33":"I F E D A B C O P bB cB dB eB TB U x iB jB"},F:{"2":"D B C H J K L W X Y Z a b c d e f g h i j k l m n o p q r s kB lB mB nB U VB PC x","132":"0 1 2 3 4 5 6 7 8 9 t u v w Q y z BB DB EB CB AB T R N"},G:{"2":"SB qB","33":"E XB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B"},H:{"2":"9B"},I:{"2":"OB G AC BC CC DC XB EC FC","132":"M"},J:{"2":"F A"},K:{"2":"A B C Q U VB x"},L:{"132":"M"},M:{"1":"N"},N:{"2":"A B"},O:{"4":"GC"},P:{"1":"IC JC KC LC TB MC","2":"G","132":"HC"},Q:{"2":"NC"},R:{"132":"OC"},S:{"1":"fB"}},B:5,C:"CSS Hyphenation"};
