"use strict";

var inspect = require("util").inspect;



function log(data)
{
	console.log( inspect(data, {depth:null, colors:true}) );
}



function logAll(data)
{
	console.log( inspect(data, {depth:null, showHidden:true, colors:true}) );
}



module.exports =
{
	log:    log,
	logAll: logAll
};
